#include <stdio.h>

#include "sci.h"
#include "hci.h"

#include "toshparm.h"

int init_sci(void);

int setlcdbright(int bright);
int printlcdbright(void);

int setcpuspeed(unsigned short cpuspeed);
int printcpuspeed(void);

int setvolume(unsigned short volume);
int printvolume(void);


int main(int argc, char* argv[])
{
	int rc;
	int printusage=0;
	
	rc=init_sci();
	if (rc) return rc;
	
	if (argc>1)
	{
		//if (strncmp(argv[1],"cpu",1)==0) printf("CPU\n");
		switch (argv[1][0])
		{
			case 'c': // CPU
				if (argc>2)
				{
					switch (argv[2][0])
					{
						case 's': // falls through
						case '0':
							rc=setcpuspeed(SCI_LOW);
						break;
						case 'f': // falls through
						case '1':
							rc=setcpuspeed(SCI_HIGH);
						break;
						default:
							printusage=1;
					}
				}
				else
				{
					rc=printcpuspeed();
				}
			break;
			case 'v': // VOL
				if (argc>2)
				{
					switch (argv[2][0])
					{
						case 'o': // falls through
						case '0':
							rc=setvolume(SCI_VOLUME_OFF);
						break;
						case 'l': // falls through
						case '1':
							rc=setvolume(SCI_VOLUME_LOW);
						break;
						case 'm': // falls through
						case '2':
							rc=setvolume(SCI_VOLUME_MEDIUM);
						break;
						case 'h': // falls through
						case '3':
							rc=setvolume(SCI_VOLUME_HIGH);
						break;
						default:
							printusage=1;
					}
				}
				else
				{
					rc=printvolume();
				}	
			break;
			case 'l': // LCD
				if (argc>2)
				{
					switch (argv[2][0])
					{
						case '0':
							rc=setlcdbright(0);
						break;
						case '1':
							rc=setlcdbright(1);
						break;
						case '2':
							rc=setlcdbright(2);
						break;
						case '3':
							rc=setlcdbright(3);
						break;
						case '4':
							rc=setlcdbright(4);
						break;
						case 'd':
							switch (argv[2][1])
							{
								case 'a': // dark
									rc=setlcdbright(0);
								break;
								case 'i': // dim
									rc=setlcdbright(1);
								break;
								default:
									printusage=1;
							}
						break;
						case 's':
							switch (argv[2][1])
							{
								case 'e': // semi-bright
									rc=setlcdbright(2);
								break;
								case 'u': // super-bright
									rc=setlcdbright(4);
								break;
								default:
									printusage=1;
							}
						case 'b': // bright
							rc=setlcdbright(3);
						break;
						default:
							printusage=1;
					}
				}
				else
				{
					rc=printlcdbright();
				}
				
			break;
			default:
				printusage=1;
		}
	}
	else
	{
		printusage=1;
	}
		
	if (printusage)
	{
		printf("Usage: %s option [value]\n"
			   "                cpu [slow|fast]\n"
			   "                lcd [dark|dim|semibright|bright|superbright]\n"
			   "                vol [off|low|medium|high]\n",argv[0]);
	}
	
	return rc;
}


int init_sci(void)
{
	int sci_version;
	int bios_version;
	int machine_id;

	if (SciSupportCheck(&sci_version)==1)
	{
		fprintf(stderr, "toshparm: computer not supported or kernel module not loaded.\n");
		return 1;
	}

	bios_version = HciGetBiosVersion();
	if (bios_version==0)
	{		
		fprintf(stderr, "toshparm: unable to get BIOS version.\n");
		return 1;
	}

	if (HciGetMachineID(&machine_id)==HCI_FAILURE)
	{
		fprintf(stderr, "toshparm: unable to get machine identification\n");
		return 1;
	}

	return 0;
}

int setlcdbright(int bright)
{
	unsigned short lcdtype;
	unsigned short lcdmode;
	SMMRegisters smmreg;
	int rc=0;

	SciOpenInterface();

	/* decide whether to make LCD brightness or backlight call */

	smmreg.ebx = SCI_LCD_BRIGHTNESS;
	smmreg.ecx = 0;
	if (SciGet(&smmreg)==SCI_SUCCESS)
	{
		lcdtype = SCI_LCD_BRIGHTNESS;
		switch (bright)
		{
			case 0:
				lcdmode=SCI_SEMI_BRIGHT;
			break;
			case 1:
				lcdmode=SCI_SEMI_BRIGHT;
			break;
			case 2:
				lcdmode=SCI_BRIGHT;
			break;
			case 3:
				lcdmode=SCI_BRIGHT;
			break;
			case 4:
				lcdmode=SCI_SUPER_BRIGHT;
			break;
			default:
				lcdmode=SCI_BRIGHT;
		}
	}
	else
	{
		lcdtype = SCI_LCD_BACKLIGHT;
		switch (bright)
		{
			case 0:
				lcdmode=SCI_BACK_DARK;
			break;
			case 1:
				lcdmode=SCI_BACK_DIM;
			break;
			case 2:
				lcdmode=SCI_BACK_SEMI;
			break;
			case 3:
				lcdmode=SCI_BACK_BRIGHT;
			break;
			case 4:
				lcdmode=SCI_BACK_BRIGHT;
			break;
			default:
				lcdmode=SCI_BACK_SEMI;
		}
	}
	smmreg.ebx = lcdtype;
	smmreg.ecx = lcdmode;

	if (SciSet(&smmreg)==SCI_FAILURE)
	{
		fprintf(stderr, "toshparm: can't set LCD brightness.\n");
		rc=1;
	}

	
	SciCloseInterface();

	return rc;
}


int printlcdbright(void)
{
	
	unsigned short lcdtype;
	SMMRegisters smmreg;
	int rc=0;

	SciOpenInterface();

	// decide whether to make LCD brightness or backlight call 

	smmreg.ebx = SCI_LCD_BRIGHTNESS;
	smmreg.ecx = 0;
	if (SciGet(&smmreg)==SCI_SUCCESS)
	{
		lcdtype = SCI_LCD_BRIGHTNESS;
	}
	else
	{
		smmreg.ebx = SCI_LCD_BACKLIGHT;
		smmreg.ecx = 0;
		if (SciGet(&smmreg)==SCI_SUCCESS)
		{
			lcdtype = SCI_LCD_BACKLIGHT;
		}
		else
		{
			fprintf(stderr, "toshparm: can't read LCD brightness.\n");
			return 1;
		}
	}
	
	printf("LCD setting: ");
	switch (lcdtype)
	{
		case SCI_LCD_BRIGHTNESS:
			switch (smmreg.ecx)
			{
				case SCI_BRIGHT:
					printf("bright");
				break;
				case SCI_SEMI_BRIGHT:
					printf("semi-bright");
				break;
				case SCI_SUPER_BRIGHT:
					printf("super-bright");
				break;
				default:
					printf("unknown %d",smmreg.ecx);
			}
		break;
		case SCI_LCD_BACKLIGHT:
			switch (smmreg.ecx)
			{
				case SCI_BACK_DARK:
					printf("dark");
				break;
				case SCI_BACK_DIM:
					printf("dim");
				break;
				case SCI_BACK_SEMI:
					printf("semi-bright");
				break;
				case SCI_BACK_BRIGHT:
					printf("bright");
				break;
				default:
					printf("unknown %d",smmreg.ecx);
			}
		break;
		default:
			printf("unknown %d lcdtype: %d",smmreg.ecx,lcdtype);
	}

	printf("\n");
		
	SciCloseInterface();

	return rc;
}


int setcpuspeed(unsigned short cpuspeed)
{
	SMMRegisters smmreg;
	int rc=0;

	SciOpenInterface();

	smmreg.ebx = SCI_PROCESSING;
	smmreg.ecx = cpuspeed;

	if (SciSet(&smmreg)==SCI_FAILURE)
	{
		fprintf(stderr, "toshparm: can't set cpu speed.\n");
		rc=1;
	}

	SciCloseInterface();

	return rc;
}

int printcpuspeed(void)
{
	SMMRegisters smmreg;
	int rc=0;

	SciOpenInterface();

	smmreg.ebx = SCI_PROCESSING;
	smmreg.ecx = 0;

	if (SciGet(&smmreg)==SCI_FAILURE)
	{
		fprintf(stderr, "toshparm: can't read cpu speed.\n");
		rc=1;
	}
	else
	{
		printf("CPU speed: ");
		switch (smmreg.ecx)
		{
			case SCI_LOW:
				printf("slow");
			break;
			case SCI_HIGH:
				printf("fast");
			break;
			default:
				printf("unknown %d",smmreg.ebx);
		}
		printf("\n");
	}

	SciCloseInterface();

	return rc;
}


int setvolume(unsigned short volume)
{
	SMMRegisters smmreg;
	int rc=0;

	SciOpenInterface();

	smmreg.ebx = SCI_SPEAKER_VOLUME;
	smmreg.ecx = volume;

	if (SciSet(&smmreg)==SCI_FAILURE)
	{
		fprintf(stderr, "toshparm: can't set speaker volume.\n");
		rc=1;
	}

	SciCloseInterface();

	return rc;
}

int printvolume(void)
{
	SMMRegisters smmreg;
	int rc=0;

	SciOpenInterface();

	smmreg.ebx = SCI_SPEAKER_VOLUME;
	smmreg.ecx = 0;

	if (SciGet(&smmreg)==SCI_FAILURE)
	{
		fprintf(stderr, "toshparm: can't read speaker volume.\n");
		rc=1;
	}
	else
	{
		printf("Volume: ");
		switch (smmreg.ecx)
		{
			case SCI_VOLUME_OFF:
				printf("off");
			break;
			case SCI_VOLUME_LOW:
				printf("low");
			break;
			case SCI_VOLUME_MEDIUM:
				printf("medium");
			break;
			case SCI_VOLUME_HIGH:
				printf("high");
			break;
			default:
				printf("unknown %d",smmreg.ebx);
		}
		printf("\n");
	}

	SciCloseInterface();

	return rc;
}

